@@ -1,27 +1,27 @@
-/*************************************************************************
- * Copyright 2010 Olivier Belanger                                        *                  
- *                                                                        * 
+/**************************************************************************
+ * Copyright 2009-2015 Olivier Belanger                                   *
+ *                                                                        *
  * This file is part of pyo, a python module to help digital signal       *
- * processing script creation.                                            *  
- *                                                                        * 
+ * processing script creation.                                            *
+ *                                                                        *
  * pyo is free software: you can redistribute it and/or modify            *
- * it under the terms of the GNU General Public License as published by   *
- * the Free Software Foundation, either version 3 of the License, or      *
- * (at your option) any later version.                                    * 
+ * it under the terms of the GNU Lesser General Public License as         *
+ * published by the Free Software Foundation, either version 3 of the     *
+ * License, or (at your option) any later version.                        *
  *                                                                        *
  * pyo is distributed in the hope that it will be useful,                 *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of         *    
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- * GNU General Public License for more details.                           *
+ * GNU Lesser General Public License for more details.                    *
  *                                                                        *
- * You should have received a copy of the GNU General Public License      *
- * along with pyo.  If not, see <http://www.gnu.org/licenses/>.           *
+ * You should have received a copy of the GNU Lesser General Public       *
+ * License along with pyo.  If not, see <http://www.gnu.org/licenses/>.   *
  *************************************************************************/
 
 #include "Python.h"
 #include <math.h>
 
-#define PYO_VERSION "0.7.0"
+#define PYO_VERSION "0.7.5"
 
 #ifndef __MYFLT_DEF
 #define __MYFLT_DEF
@@ -38,6 +38,8 @@
 #define TYPE__FII "|fii"
 #define TYPE_F_II "f|ii"
 #define TYPE__FFF "|fff"
+#define TYPE_F_FFF "f|fff"
+#define TYPE_O_FIFFI "O|fiffi"
 #define TYPE_O_F "O|f"
 #define TYPE_O_FO "O|fO"
 #define TYPE__OF "|Of"
@@ -120,6 +122,8 @@
 #define TYPE__FII "|dii"
 #define TYPE_F_II "d|ii"
 #define TYPE__FFF "|ddd"
+#define TYPE_F_FFF "d|ddd"
+#define TYPE_O_FIFFI "O|diddi"
 #define TYPE_O_F "O|d"
 #define TYPE_O_FO "O|dO"
 #define TYPE__OF "|Od"
@@ -305,6 +309,7 @@ extern PyTypeObject CtlScanType;
 extern PyTypeObject CtlScan2Type;
 extern PyTypeObject MidiNoteType;
 extern PyTypeObject NoteinType;
+extern PyTypeObject NoteinTrigType;
 extern PyTypeObject BendinType;
 extern PyTypeObject TouchinType;
 extern PyTypeObject PrograminType;
@@ -339,6 +344,7 @@ extern PyTypeObject SndTableType;
 extern PyTypeObject DataTableType;
 extern PyTypeObject NewTableType;
 extern PyTypeObject TableRecType;
+extern PyTypeObject TableWriteType;
 extern PyTypeObject TableRecTimeStreamType;
 extern PyTypeObject TableMorphType;
 extern PyTypeObject TrigTableRecType;
@@ -398,6 +404,7 @@ extern PyTypeObject ScaleType;
 extern PyTypeObject CentsToTranspoType;
 extern PyTypeObject TranspoToCentsType;
 extern PyTypeObject MToFType;
+extern PyTypeObject FToMType;
 extern PyTypeObject MToTType;
 extern PyTypeObject M_SinType;
 extern PyTypeObject M_CosType;
@@ -464,6 +471,20 @@ extern PyTypeObject ComplexResType;
 extern PyTypeObject STReverbType;
 extern PyTypeObject STRevType;
 extern PyTypeObject Pointer2Type;
+extern PyTypeObject CentroidType;
+extern PyTypeObject AttackDetectorType;
+extern PyTypeObject SmoothDelayType;
+extern PyTypeObject TrigBursterType;
+extern PyTypeObject TrigBurstType;
+extern PyTypeObject TrigBurstTapStreamType;
+extern PyTypeObject TrigBurstAmpStreamType;
+extern PyTypeObject TrigBurstDurStreamType;
+extern PyTypeObject TrigBurstEndStreamType;
+extern PyTypeObject ScopeType;
+extern PyTypeObject PeakAmpType;
+extern PyTypeObject MainParticleType;
+extern PyTypeObject ParticleType;
+extern PyTypeObject AtanTableType;
 
 /* Constants */
 #define E M_E
@@ -501,6 +522,8 @@ extern PyTypeObject Pointer2Type;
 #define FREEVERB_ID 24
 #define XNOISEDUR_ID 25
 #define URN_ID 26
+#define GRANULE_ID 27
+#define MAINPARTICLE_ID 28
 /* Do not forget to modify Server_generateSeed function */
 
 /* object headers */
@@ -517,8 +540,9 @@ extern PyTypeObject Pointer2Type;
     Stream *add_stream; \
     int bufsize; \
     int nchnls; \
+    int ichnls; \
     double sr; \
-    MYFLT *data; 
+    MYFLT *data;
 
 #define pyo_table_HEAD \
     PyObject_HEAD \
@@ -543,7 +567,7 @@ extern PyTypeObject Pointer2Type;
     Py_VISIT(self->mul); \
     Py_VISIT(self->mul_stream); \
     Py_VISIT(self->add); \
-    Py_VISIT(self->add_stream);    
+    Py_VISIT(self->add_stream);
 
 #define pyo_table_VISIT \
     if (PyServer_get_server() != NULL) \
@@ -616,6 +640,7 @@ extern PyTypeObject Pointer2Type;
     self->bufsize = PyInt_AsLong(PyObject_CallMethod(self->server, "getBufferSize", NULL)); \
     self->sr = PyFloat_AsDouble(PyObject_CallMethod(self->server, "getSamplingRate", NULL)); \
     self->nchnls = PyInt_AsLong(PyObject_CallMethod(self->server, "getNchnls", NULL)); \
+    self->ichnls = PyInt_AsLong(PyObject_CallMethod(self->server, "getIchnls", NULL)); \
     self->data = (MYFLT *)realloc(self->data, (self->bufsize) * sizeof(MYFLT)); \
     for (i=0; i<self->bufsize; i++) \
         self->data[i] = 0.0; \
@@ -644,13 +669,14 @@ extern PyTypeObject Pointer2Type;
         PyErr_SetString(PyExc_TypeError, "The data must be a list of floats."); \
         return PyInt_FromLong(-1); \
     } \
-    self->size = PyList_Size(arg)-1; \
+    self->size = PyList_Size(arg); \
     self->data = (MYFLT *)realloc(self->data, (self->size+1) * sizeof(MYFLT)); \
     TableStream_setSize(self->tablestream, self->size+1); \
  \
-    for (i=0; i<(self->size+1); i++) { \
+    for (i=0; i<(self->size); i++) { \
         self->data[i] = PyFloat_AS_DOUBLE(PyNumber_Float(PyList_GET_ITEM(arg, i))); \
     } \
+    self->data[self->size] = self->data[0]; \
     TableStream_setData(self->tablestream, self->data); \
  \
     Py_INCREF(Py_None); \
@@ -693,7 +719,115 @@ extern PyTypeObject Pointer2Type;
     } \
     self->data[self->size] = self->data[0]; \
     Py_RETURN_NONE; \
-    
+
+#define TABLE_ADD \
+    int i, tabsize; \
+    MYFLT x = 0.0; \
+    MYFLT *list = NULL; \
+    PyObject *table = NULL; \
+    if (PyNumber_Check(arg)) { \
+        x = PyFloat_AsDouble(PyNumber_Float(arg)); \
+        for (i=0; i<self->size; i++) { \
+            self->data[i] += x; \
+        } \
+    } \
+    else if ( PyObject_HasAttrString((PyObject *)arg, "getTableStream") == 1 ) { \
+        Py_XDECREF(table); \
+        table = PyObject_CallMethod((PyObject *)arg, "getTableStream", ""); \
+        list = TableStream_getData((TableStream *)table); \
+        tabsize = TableStream_getSize((TableStream *)table); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] += list[i]; \
+        } \
+    } \
+    else if (PyList_Check(arg)) { \
+        tabsize = PyList_Size(arg); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] += PyFloat_AS_DOUBLE(PyNumber_Float(PyList_GET_ITEM(arg, i))); \
+        } \
+    } \
+ \
+    self->data[self->size] = self->data[0]; \
+ \
+    Py_INCREF(Py_None); \
+    return Py_None; \
+
+#define TABLE_SUB \
+    int i, tabsize; \
+    MYFLT x = 0.0; \
+    MYFLT *list = NULL; \
+    PyObject *table = NULL; \
+    if (PyNumber_Check(arg)) { \
+        x = PyFloat_AsDouble(PyNumber_Float(arg)); \
+        for (i=0; i<self->size; i++) { \
+            self->data[i] -= x; \
+        } \
+    } \
+    else if ( PyObject_HasAttrString((PyObject *)arg, "getTableStream") == 1 ) { \
+        Py_XDECREF(table); \
+        table = PyObject_CallMethod((PyObject *)arg, "getTableStream", ""); \
+        list = TableStream_getData((TableStream *)table); \
+        tabsize = TableStream_getSize((TableStream *)table); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] -= list[i]; \
+        } \
+    } \
+    else if (PyList_Check(arg)) { \
+        tabsize = PyList_Size(arg); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] -= PyFloat_AS_DOUBLE(PyNumber_Float(PyList_GET_ITEM(arg, i))); \
+        } \
+    } \
+ \
+    self->data[self->size] = self->data[0]; \
+ \
+    Py_INCREF(Py_None); \
+    return Py_None; \
+
+#define TABLE_MUL \
+    int i, tabsize; \
+    MYFLT x = 0.0; \
+    MYFLT *list = NULL; \
+    PyObject *table = NULL; \
+    if (PyNumber_Check(arg)) { \
+        x = PyFloat_AsDouble(PyNumber_Float(arg)); \
+        for (i=0; i<self->size; i++) { \
+            self->data[i] *= x; \
+        } \
+    } \
+    else if ( PyObject_HasAttrString((PyObject *)arg, "getTableStream") == 1 ) { \
+        Py_XDECREF(table); \
+        table = PyObject_CallMethod((PyObject *)arg, "getTableStream", ""); \
+        list = TableStream_getData((TableStream *)table); \
+        tabsize = TableStream_getSize((TableStream *)table); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] *= list[i]; \
+        } \
+    } \
+    else if (PyList_Check(arg)) { \
+        tabsize = PyList_Size(arg); \
+        if (self->size < tabsize) \
+            tabsize = self->size; \
+        for (i=0; i<tabsize; i++) { \
+            self->data[i] *= PyFloat_AS_DOUBLE(PyNumber_Float(PyList_GET_ITEM(arg, i))); \
+        } \
+    } \
+ \
+    self->data[self->size] = self->data[0]; \
+ \
+    Py_INCREF(Py_None); \
+    return Py_None; \
+
 #define SET_TABLE \
     int i; \
     if (arg == NULL) { \
@@ -990,10 +1124,10 @@ extern PyTypeObject Pointer2Type;
     if (! PyArg_ParseTupleAndKeywords(args, kwds, TYPE_F_I, kwlist, &val, &pos)) \
         return PyInt_FromLong(-1); \
  \
-    if (pos >= self->size) { \
-        PyErr_SetString(PyExc_TypeError, "position outside of table boundaries!."); \
-        return PyInt_FromLong(-1); \
-    } \
+    if (pos >= self->size) \
+        pos = self->size - 1; \
+    else if (pos < 0) \
+        pos = 0; \
  \
     self->data[pos] = val; \
  \
@@ -1185,7 +1319,7 @@ extern PyTypeObject Pointer2Type;
     (*self->mode_func_ptr)(self); \
  \
     Py_INCREF(Py_None); \
-    return Py_None; 
+    return Py_None;
 
 #define SET_ADD \
     PyObject *tmp, *streamtmp; \
@@ -1216,7 +1350,7 @@ extern PyTypeObject Pointer2Type;
     (*self->mode_func_ptr)(self); \
 \
     Py_INCREF(Py_None); \
-    return Py_None; 
+    return Py_None;
 
 #define SET_SUB \
     PyObject *tmp, *streamtmp; \
@@ -1247,7 +1381,7 @@ extern PyTypeObject Pointer2Type;
     (*self->mode_func_ptr)(self); \
  \
     Py_INCREF(Py_None); \
-    return Py_None; 
+    return Py_None;
 
 #define SET_DIV \
     PyObject *tmp, *streamtmp; \
@@ -1281,7 +1415,7 @@ extern PyTypeObject Pointer2Type;
     (*self->mode_func_ptr)(self); \
  \
     Py_INCREF(Py_None); \
-    return Py_None; 
+    return Py_None;
 
 /* Multiply, Add, inplace_multiply & inplace_add */
 #define MULTIPLY \
@@ -1408,7 +1542,7 @@ extern PyTypeObject Pointer2Type;
         self->data[i] = 0; \
     } \
     Py_INCREF(Py_None); \
-    return Py_None;    
+    return Py_None;
 
 /* Post processing (mul & add) macros */
 #define POST_PROCESSING_II \
@@ -1444,7 +1578,7 @@ extern PyTypeObject Pointer2Type;
         old = self->data[i]; \
         val = mul * old + add[i]; \
         self->data[i] = val; \
-    } 
+    }
 
 #define POST_PROCESSING_AA \
     MYFLT old, val; \
@@ -1494,7 +1628,7 @@ extern PyTypeObject Pointer2Type;
         old = self->data[i]; \
         val = mul * old - add[i]; \
         self->data[i] = val; \
-    } 
+    }
 
 #define POST_PROCESSING_AREVA \
     MYFLT old, val; \
@@ -1520,4 +1654,3 @@ extern PyTypeObject Pointer2Type;
         val = old / tmp - add[i]; \
         self->data[i] = val; \
     }
-
